# gzv.py - an user interface to generate-zooming-video
#
# Copyright (C) 2008  Lincoln de Sousa <lincoln@minaslivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import gtk
import gtk.glade
import math
import cairo

_ = lambda x:x

class Ball(object):
    DEFAULT_WIDTH = 10

    def __init__(self, x, y, r, name='', position=0):
        self.position = position
        self.x = x
        self.y = y
        self.radios = r
        self.name = name

class BallManager(list):
    def __init__(self, *args, **kwargs):
        super(BallManager, self).__init__(*args, **kwargs)

    def save_to_file(self, path):
        target = open(path, 'w')
        for i in self:
            target.write('%d,%d %d %s\n' % (i.x, i.y, i.radios, i.name))
        target.close()

class GladeLoader(object):
    def __init__(self, fname, root=''):
        self.ui = gtk.glade.XML(fname, root)
        self.ui.signal_autoconnect(self)

    def get_widget(self, wname):
        return self.ui.get_widget(wname)

    # little shortcut
    wid = get_widget

    # glade callbacks

    def gtk_widget_show(self, widget, *args):
        widget.show()
        return True

    def gtk_widget_hide(self, widget, *args):
        widget.hide()
        return True

    def gtk_main_quit(self, *args):
        gtk.main_quit()

    def gtk_main(self, *args):
        gtk.main()

class NewProject(GladeLoader):
    def __init__(self, parent=None):
        super(NewProject, self).__init__('gzv.glade', 'new-project')
        if parent:
            self.wid('new-project').set_transient_for(parent)

class Gzv(GladeLoader):
    def __init__(self):
        super(Gzv, self).__init__('gzv.glade', 'main-window')
        self.window = self.wid('main-window')
        self.window.connect('delete-event', lambda *x: gtk.main_quit())

        self.evtbox = self.wid('eventbox')
        self.evtbox.connect('button-press-event', self.button_press)
        self.evtbox.connect('button-release-event', self.button_release)
        self.evtbox.connect('motion-notify-event', self.motion_notify)

        self.model = gtk.ListStore(int, str)
        self.treeview = self.wid('treeview')
        self.treeview.set_model(self.model)

        self.draw = self.wid('draw')
        self.draw.connect('expose-event', self.expose_draw)

        # FIXME: Hardcoded.
        self.image = 'skol.jpg'
        self.balls = self.load_balls_from_file('xxx')
        self.load_balls_to_treeview()

        # this *MUST* be called *AFTER* load_balls_to_treeview
        self.setup_treeview()

        self.ball_width = Ball.DEFAULT_WIDTH
        self.selecting = False
        self.start_x = -1
        self.start_y = -1

    def setup_treeview(self):
        self.model.connect('rows-reordered', self.on_rows_reordered)

        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Position'), renderer, text=0)
        self.treeview.append_column(column)

        renderer = gtk.CellRendererText()
        renderer.connect('edited', self.on_cell_edited)
        renderer.set_property('editable', True)
        column = gtk.TreeViewColumn(_('Name'), renderer, text=1)
        self.treeview.append_column(column)

    def on_rows_reordered(self, *args):
        print 

    def on_cell_edited(self, *args):
        print args

    def new_project(self, button):
        dialog = NewProject(self.window).wid('new-project')

        # This '1' was defined in the glade file
        if dialog.run() == 1:
            pass
        dialog.destroy()

    def open_file_chooser(self, button):
        fc = gtk.FileChooserDialog(_('Choose an image'), self,
                                   buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                            gtk.STOCK_OK, gtk.RESPONSE_OK))
        if fc.run() == gtk.RESPONSE_OK:
            self.image = fc.get_filename()

        fc.destroy()

    def load_balls_to_treeview(self):
        model = self.treeview.get_model()
        for i in self.balls:
            model.append([i.position, i.name])

    def load_balls_from_file(self, fname):
        balls = BallManager()
        for index, line in enumerate(file(fname)):
            if not line:
                continue
            pos, radios, name = line.split()
            x, y = pos.split(',')
            balls.append(Ball(int(x), int(y), int(radios), name, index))
        return balls

    def expose_draw(self, draw, event):
        if not self.image:
            return

        # loading the picture image and getting some useful
        # information to draw it in the widget's background
        img = gtk.gdk.pixbuf_new_from_file(self.image)
        pixels = img.get_pixels()
        rowstride = img.get_rowstride()
        width = img.get_width()
        height = img.get_height()
        gc = draw.style.black_gc

        # sets the correct size of the eventbox, to show the scrollbar
        # when needed.
        self.evtbox.set_size_request(width, height)

        # drawing the picture in the background of the drawing area,
        # this is really important.
        draw.window.draw_rgb_image(gc, 0, 0, width, height,
                                   'normal', pixels, rowstride,
                                   0, 0)

        # this call makes the ball being drown be shown correctly.
        self.draw_current_ball()

        # drawing other balls stored in the self.balls list.
        ctx = draw.window.cairo_create()
        ctx.fill()

        ctx.set_line_width(10.0)
        ctx.set_source_rgba (0.5, 0.0, 0.0, 0.4)

        for i in self.balls:
            ctx.arc(i.x, i.y, i.radios, 0, 64*math.pi)

        ctx.fill()
        ctx.stroke()

    def draw_current_ball(self):
        if self.start_x < 0:
            return
        ctx = self.draw.window.cairo_create()
        ctx.arc(self.start_x, self.start_y, self.ball_width, 0, 64*math.pi)
        ctx.set_source_rgba (0.5, 0.0, 0.0, 0.4)
        ctx.fill()

    def button_press(self, widget, event):
        if event.button == 1:
            self.selecting = True
            self.start_x = event.x
            self.start_y = event.y
            self.last_x = event.x

    def button_release(self, widget, event):
        if event.button == 1:
            self.selecting = False
            self.finish_drawing()

    def motion_notify(self, widget, event):
        self.draw.queue_draw()

        if event.x > self.last_x:
            self.ball_width += 2
        else:
            self.ball_width -= 2

        self.last_x = event.x

    def finish_drawing(self):
        self.draw_current_ball()
        self.ball_width = Ball.DEFAULT_WIDTH

if __name__ == '__main__':
    Gzv().window.show_all()
    gtk.main()
